getwd()

#import data
train.data <- read.csv("data/titanic/train.csv", stringsAsFactors = F)
test.data <- read.csv("data/titanic/test.csv", stringsAsFactors = F)

##################################
######  EXPLORING THE DATA  ######
##################################

# Let's start by examining structure of the training dataset
# Note: description of all the varibles is available at the Kaggle website:
# http://www.kaggle.com/c/titanic-gettingStarted/data
str(train.data)

# We can also take a look at the first couple of rows
head(train.data)

# Let's see the structure of the test dataset, as well
# the structure is almost exactly the same as of the training set, 
# except that the Survived column is missing
str(test.data)

# Let's now check if the data is complete, i.e., if there is some missing values.
# The easiest way to get a high level view on the data completeness is to visualize the data
# To that end, we'll use the Amelia package; first, we install the package  
install.packages("Amelia")
# and the we load it 
library(Amelia)
# we now structure the display area to show two plots in the same row  
par(mfrow = c(1,2))
# printing graphs of missing data for the training and the test sets
missmap(obj = train.data, main = "Missing data in the training set", legend = F)
missmap(obj = test.data, main = "Missing data in the test set", legend = F)
# reverting plotting area to the default (one plot per row)
par(mfrow = c(1,1))

# there's a lot of missing data for the Age variable; let's see that more precisely
summary( train.data$Age )
summary( test.data$Age )

# there is also one missing Fare value in the test set; let's see which one
which( is.na(test.data$Fare) )
# it's the 153rd instance; let's see it 
test.data[153, ]

# we previously noticed that in some observations the value for the Cabin variable is missing;
# that was not visible in the previous plots, as the detection of missing values was 
# based on the NA values
# let's check how many missing values we have for the Cabin in both datasets
length( which(train.data$Cabin == "") )
length( which(test.data$Cabin == "") )

# some of these missing values are due to the fact that passangers from the
# 2nd and the 3rd class did not have a cabin; however, those from the 1st class
# must have had a cabin; let's check
nrow(train.data[train.data$Cabin == "" & train.data$Pclass == 1,])
# for 40 1st class passengers in the training set the cabin value is missing
nrow(test.data[test.data$Cabin == "" & test.data$Pclass == 1,])
# and for 27 1st class passangers from the test set

# so, let's replace the missing Cabin values for 1st class passangers with NAs 
# and plot the missing data again
train.data[ (train.data$Cabin == "") & (train.data$Pclass == 1), "Cabin"] <- NA
test.data[ (test.data$Cabin == "") & (test.data$Pclass == 1), "Cabin"] <- NA
par(mfrow = c(1,2))
missmap(obj = train.data, main = "Missing data in the training set", legend = F)
missmap(obj = test.data, main = "Missing data in the test set", legend = F)
par(mfrow = c(1,1))

# Let's replace the missing Fare value with the mean value of the Fare variable
# for the passengers of the same Pclass 
# this is a typical technique used for imputing the missing values 
# (imputation is the process of replacing missing data with substituted values) 
missing.fare.index <- which( is.na(test.data$Fare) )
missing.fare.pclass <- test.data$Pclass[missing.fare.index]
mean.fare <- mean(x = test.data$Fare[test.data$Pclass == missing.fare.pclass], na.rm = T)
test.data$Fare[which(is.na(test.data$Fare))] <- mean.fare

# since we should predict the survival of the passangers, 
# let's see what it looks like in the training set
table( train.data$Survived )
# we can see this as percentages
prop.table( table(train.data$Survived) )

# It's well-known that in disasters woman and children are often the first to be rescued
# Let's check if that was the case in the Titanic disaster
# We'll start by looking at the survival based on the gender;
# first, let's see the proportion of males and females in the dataset
train.data$Sex <- factor(train.data$Sex)
summary( train.data$Sex )
# let's see the survival counts based on the gender
table(train.data$Sex, train.data$Survived )
# and the proportions
prop.table( table(train.data$Sex, train.data$Survived) )
# but, the table gives percentages with respect to the entire population (all the passangers)
# we want gender-wise proportions; in this case those are row-wise proportions
prop.table(x = table(train.data$Sex, train.data$Survived), margin = 1) #1 indicates the 1st dimension, which is row

# This is quite informative, but we might better appreciate the data if it is presented visually
# For drawing graphs, we'll use the ggplot2 library
library(ggplot2)

# recall that the Survived varible is of the type int; we can check that with: 
class( train.data$Survived )
# for plotting the survival against the gender of the passengers, we need to 
# transform the Survived variable into a factor variable (like Enum in Java)
train.data$Survived <- factor(x = train.data$Survived, levels = c(0,1), labels = c("No", "Yes"))

# plot survival against the gender 
ggplot(train.data, aes(x = Sex, fill = Survived)) +
  geom_histogram(width = 0.35) +
  xlab("Gender") +
  ylab("Total Count") +
  labs(fill = "Survived")

# Before inspecting if/how age have affected the chances for survival, let's quickly take a look
# at the potential impact of the passanger class (1st, 2nd or 3rd)
# Again, we first have to transform the variable into a factor
train.data$Pclass <- factor(x = train.data$Pclass, 
                            levels = c(1,2,3), 
                            labels = c("1st", "2nd", "3rd"))

# let's now plot the survival against gender and the passanger class
ggplot(train.data, aes(x = Sex, fill = Survived)) +
  facet_wrap(~Pclass) +
  geom_histogram(width = 0.35) +
  xlab("Gender") +
  ylab("Total Count") +
  labs(fill = "Survived") +
  ggtitle("Passanger class")

# let's also inspect if the place of embarkment (the Embarked variable)
# affect the survival
train.data$Embarked <- factor(train.data$Embarked) 
# see the distribution of values
table(train.data$Embarked)
# plot it agains the survival
ggplot(train.data, aes(x = Embarked, fill = Survived)) +
  geom_histogram(width = 0.35) +
  xlab("Place of embarkment") +
  ylab("Total Count") +
  labs(fill = "Survived")


###################################
######  FEATURE ENGINEERING  ######
###################################
# When creating new features (attributes) to be used for prediction purposes, 
# we need to base those features on the data from both the training and the test set, 
# so that the features are available for training the prediction model, 
# and making predictions on the unseen test data.
# Hence, we will merge the training and the test set and develop features on the merged data

# first, assure that the training and the test sets have exactly the same structure
# to that end, we add the Survived column to the test data
test.data$Survived <- factor(NA, # NA is like null in Java
                             levels = c(0,1), 
                             labels = c("No", "Yes")) 
# transform the Pclass variable in the test set into factors, since we've done 
# that in the training set (the structure should be exactly the same)  
test.data$Pclass <- factor(x = test.data$Pclass, levels = c(1,2,3), labels = c("1st", "2nd", "3rd"))
test.data$Sex <- factor(test.data$Sex)
test.data$Embarked <- factor(test.data$Embarked)
# now, we can merge the two datasets
merged.data <- rbind(train.data, test.data)


# Recall that the Age variable has a lot of missing values 
nrow( merged.data[ is.na(merged.data$Age), ]) / dim( merged.data )[1]
# 20% of all observations are missing the Age value; none of the imputation techniques would 
# produce trustworthy replacements; so, we need to find another way to determine 
# or at least to approximate age groups; 
# we'll do that by making use of the Name variable

# To start, let's first inspect the values of this variable
merged.data[1:10, "Name"] # names of the first ten passangers 

# an observation regarding the Name variable:
# it consists of surname, title, first name, and in some cases additional name 
#   (probably maiden name of married woman)
# So, the idea is to use the title of a person as a rough proxy for his/her age

# We need to extract title from the Name variable; to do that, we'll split 
# the Name string using "," and "." as delimiters 
# lets' try it first
strsplit(x = merged.data$Name[1], split = "[,|.]")
# we get a list of vectors, where each vector consists of pieces of a person's name
# to extract the title, we need to first get the vector (made out of the person's name)
strsplit(x = merged.data$Name[1], split = "[,|.]")[[1]]
# and then, take the second element of that vector:
strsplit(x = merged.data$Name[1], split = "[,|.]")[[1]][2]
# you might have noticed a space before the title, we'll remove that quickly, but before that,
# we'll apply this procedure to all the rows in the merged dataset to create a new feature:
merged.data$Title <- sapply(X = merged.data$Name, 
                            FUN = function(x) { strsplit(x, "[,|.]")[[1]][2] })

# now, let's remove that leading blank space
merged.data$Title <- sapply(X = merged.data$Title, 
                            FUN = function(x) { substr(x, start = 2, stop = nchar(x))} )

# we can now inspect different kinds of titles we have in the dataset
table( merged.data$Title )
# there are some rarely occuring titles that won't be much usefull for creating a model; 
# so, we'll aggregate those titles into broader categories
ladies <- c("Dona", "Lady", "Mme", "Mrs", "the Countess")
merged.data$Title[ merged.data$Title %in% ladies ] <- "Mrs" # the %in% operator checks to see if a value is an element of the given vector 

misses = c("Miss", "Mlle")
merged.data$Title[ merged.data$Title %in% misses ] <- "Ms"

gentelman <- c("Capt", "Col", "Don", "Dr", "Major", "Mr", "Rev", "Sir")
merged.data$Title[ merged.data$Title %in% gentelman ] <- "Mr"

masters <- c("Master", "Jonkheer")
merged.data[ merged.data$Title %in% masters, "Title"] <- "Master"

# let's see how titles are distributed after the aggregation
table( merged.data$Title )
prop.table( table( merged.data$Title ) )

# the high number of Ms titles indicate that it probably does not properly
# reflect the age - it includes unmarried women of various ages
# let's try to fix that by using the Age varible where available
nrow( merged.data[ !is.na(merged.data$Age) & merged.data$Title == "Ms",])
# we have 213 cases where Age is available for observations with the Ms title
# similarly, there is high percentage of Mr title and a very small percentage of Master
# we can make use of the Age variable, when available, to try to improve this
nrow( merged.data[ !is.na(merged.data$Age) & merged.data$Title == "Mr",])
# even better situation is with the Mr title (605 observations)

# if Age is avaialble and greater than 18, and the Title is 'Ms', change the title to 'Mrs'
nrow( merged.data[ !is.na(merged.data$Age) & 
                     (merged.data$Age >= 18) & 
                     merged.data$Title == "Ms",])
# 146 observations can be changed from "Ms" to "Mrs"
merged.data$Title[(merged.data$Age >= 18) & (merged.data$Title == "Ms")] <- "Mrs"
# if Age is avaialble and less than 18, and the Title is 'Mr', change the title to 'Master'
nrow( merged.data[ !is.na(merged.data$Age) & 
                     (merged.data$Age < 18) & 
                     merged.data$Title == "Mr",])
# 29 observations can be changed from "Mr" to "Master"
merged.data$Title[(merged.data$Age < 18) & (merged.data$Title == "Mr")] <- "Master"
# let's check the title proportions again
table( merged.data$Title )
prop.table( table( merged.data$Title ) )
# this looks more realistic

# we'll transform Title into a factor variable, so that we can better use it
# for data exploration and prediction purposes
merged.data$Title <- factor(merged.data$Title)
str(merged.data$Title)

# let's plot survival against the title 
# note that we are using only the first 891 instances in the merged dataset as these are
# instances from the training set for which we know the outcome (i.e., survival)
ggplot(merged.data[1:891, ], aes(x = Title, fill = Survived)) +
  geom_histogram(width = 0.35) +
  xlab("Title") +
  ylab("Total Count") +
  labs(fill = "Survived")

# Let's also see how economic status (approximated via passenger class) 
# affects this distribution
ggplot(merged.data[1:891, ], aes(x = Title, fill = Survived)) +
  facet_wrap(~Pclass) +
  geom_histogram(width = 0.35) +
  xlab("Title") +
  ylab("Total Count") +
  labs(fill = "Survived") +
  ggtitle("Passanger class")
# obviously the economic status plays a significant role

# we'll introduce an additional feature related to the number of family members 
# one is travelling with
# to that end, we'll simply add SibSp (the number of siblings and spouses) and 
# Parch (the number of parents and children the passengers had with them)
merged.data$FamilySize <- merged.data$SibSp + merged.data$Parch
summary( merged.data$FamilySize )
table( merged.data$FamilySize )
# obviously, majority of people are travelling alone
# it can be also observed that those who travelled with 3+ family members were not that numerous
# so, we can aggregate instances with 3+ family members:
merged.data$FamilySize[ merged.data$FamilySize > 3 ] <- 3
table( merged.data$FamilySize )
# and turn FamilySize into a factor
merged.data$FamilySize <- factor(x = merged.data$FamilySize, levels = c(0,1,2,3), 
                                 labels = c("Alone", "1", "2", "3+"))

# Let's see how this new feature affects the survival prospects
ggplot(merged.data[1:891, ], aes(x = Title, fill = Survived)) +
  facet_wrap(~FamilySize) +
  geom_histogram(width = 0.35) +
  xlab("Title") +
  ylab("Total Count") +
  labs(fill = "Survived") +
  ggtitle("Family Size")
